.page.auth img.logo {
    display: none !important;
}

.page.auth img.logo.visible {
    display: inline-block !important;
}

#tab-headers .active {
    background: #aa2328 !important;
}

#tab-headers .active i::before {
    color: #eee !important;
}

.ui-tabs-active.ui-state-active {
    background: #aa2328 !important;
}

.main-sections-category .active {
    background: #aa2328 !important;
    color: #eee !important;
    text-shadow: none !important;
}

.main-sections-category .active i::before {
    color: #eee !important;
    text-shadow: none !important;
}

.control.progressbar div.fill {
    background: #aa2328 !important;
}
