// Generated by CoffeeScript 1.7.1
(function() {
  var Feedback;

  Feedback = (function() {
    function Feedback() {}

    Feedback.prototype.configure = function(enabled, os, version, edition) {
      this.enabled = enabled;
      this.os = os;
      this.version = version;
      this.edition = edition;
      return mixpanel.init(this.token);
    };

    Feedback.prototype.emit = function(evt, params) {
      var e;
      if (this.enabled) {
        if (params == null) {
          params = {};
        }
        params.os = this.os;
        params.version = this.version;
        params.edition = this.edition;
        try {
          return mixpanel.track(evt, params);
        } catch (_error) {
          e = _error;
        }
      }
    };

    return Feedback;

  })();

  window.Feedback = new Feedback();

  window.Feedback.token = "b5e6ddf58b2d02245a7a19005d1cec48";

}).call(this);
