html.mode-tablet #header .container {
  margin: 0;
}
html.mode-tablet .control.main-sections-root > .sidebar .tab {
  padding: 5px 10px;
  font-size: 12px;
  margin: 0;
  border-radius: 3px;
}
html.mode-tablet .control.main-page > .content-wr {
  padding: 0;
}
html.mode-tablet .control.main-page > .content-wr > .content {
  margin: 0 0 60px;
}
html.mode-tablet .control.main-page > .content-wr > .content .sidebar {
  border-right: 1px solid #ccc;
  width: 149px;
  padding: 0;
}
html.mode-tablet .control.main-page > .content-wr > .content .sidebar a {
  border-radius: 0;
}
html.mode-tablet .control.main-page > .content-wr > .content .sidebar .main-sections-category > div:first-child {
  margin: 10px 0 0 8px;
  font-size: 14px;
  border-bottom: 1px solid #ddd;
}
html.mode-tablet .control.main-page > .content-wr > .content .main {
  padding-top: 10px;
}
