@orange: #ed930d;

@fontFamily: "Open Sans", sans-serif;
@fontSize: 11px;
@fontSizeSmaller: 11px;

@headerHeight: 32px;

@tableCell: #fcfcfc;
@tableBorder: #f0f0f0;

@controlText: #7c8c99; //#566675;
@controlTextLighter: lighten(@controlText, 20%);

@controlHeight: 30px;
@controlWidth: 120px;
@controlWidth2: 160px;
@controlWidth3: 240px;

@controlBorder: #eee;
@buttonColor: #fcfcfc;