#header .logo .text {
    display: none;
}

#header .logo {
    width: 160px;
    margin-top: 3px;
    height: 25px;
    float: left;

    background: url("/ajenti:static/custom_branding_kaboa/logo.png");
    background-size: contain;
    background-repeat: no-repeat;
}

.page.auth img.logo {
    display: none !important;
}

.page.auth img.logo.visible {
    display: inline-block !important;
}

#tab-headers .active {
    background: #faa51a !important;
}

#tab-headers .active i::before {
    color: #eee !important;
}

.ui-tabs-active.ui-state-active {
    background: #faa51a !important;
}

.main-sections-category .active {
    background: #faa51a !important;
    color: #eee !important;
    text-shadow: none !important;
}

.main-sections-category .active i::before {
    color: #eee !important;
    text-shadow: none !important;
}

.control.progressbar div.fill {
    background: #faa51a !important;
}
